﻿using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Threading.Tasks;
using AdmCloudInterface.Models;
using System.Net.Http.Headers;
using AdmCloudInterface.Config;

namespace AdmCloudInterface.Repositories
{
    public class CurrenciesRepository : BaseRepository
    {
        private string _email;
        private string _password;
        private string _company;
        private string _role;
        private Guid _appid;
        private ServerEnum _server;
        private string _apiBase;

        public CurrenciesRepository(string email, string password, string company,
            string role, Guid appid, ServerEnum server = ServerEnum.Production)
        {
            _email = email;
            _password = password;
            _company = company;
            _role = role;
            _appid = appid;
            _server = server;

            if (_server == ServerEnum.Development)
                _apiBase = Config.Constants.API_BASE_DEVELOPMENT;
            else if (_server == ServerEnum.Staging)
                _apiBase = Config.Constants.API_BASE_STAGING;
            else if (_server == ServerEnum.Beta)
                _apiBase = Config.Constants.API_BASE_BETA;
            else
                _apiBase = Config.Constants.API_BASE;
        }

        public async Task<List<Currency>> GetListAsync(int skip = 0)
        {
            //Llamar Web Service
            var client = new HttpClient();

            string url = $"{_apiBase}Currencies?skip={skip}&company={_company}&role={_role}&appid={_appid}";

            client.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Basic", GetAuthHeader(_email, _password));

            var response = await client.GetAsync(url);
            if (response.StatusCode != System.Net.HttpStatusCode.OK)
            {
                var apiBadResponseString = await response.Content.ReadAsStringAsync();
                var errorObject = new ApiResponse();
                errorObject.message = "Ha ocurrido un error " + response.StatusCode;
                throw new ApplicationException(errorObject.message);
            }

            var apiResponseString = await response.Content.ReadAsStringAsync();

            var returnedMsg = Newtonsoft.Json.JsonConvert.DeserializeObject<ApiResponse>(apiResponseString);

            if (!returnedMsg.success)
                throw new Exception(returnedMsg.message);

            string invoicesString = returnedMsg.data.ToString();
            var list = Newtonsoft.Json.JsonConvert.DeserializeObject<List<Models.Currency>>(invoicesString);

            //Retornar
            return list;
        }
    }
}
