﻿using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Threading.Tasks;
using AdmCloudInterface.Models;
using System.Net.Http.Headers;
using AdmCloudInterface.Config;
using System.Text;

namespace AdmCloudInterface.Repositories
{
    public class LoginRepository : BaseRepository
    {
        private string _email;
        private string _password;
        private string _company;
        private Guid _appid;
        private ServerEnum _server;
        private string _apiBase;

        public LoginRepository(string email, string password, string company,
            Guid appid, ServerEnum server = ServerEnum.Production)
        {
            _email = email;
            _password = password;
            _company = company;
            _appid = appid;
            _server = server;

            if (_server == ServerEnum.Development)
                _apiBase = Config.Constants.API_BASE_DEVELOPMENT;
            else if (_server == ServerEnum.Staging)
                _apiBase = Config.Constants.API_BASE_STAGING;
            else if (_server == ServerEnum.Beta)
                _apiBase = Config.Constants.API_BASE_BETA;
            else 
                _apiBase = Config.Constants.API_BASE;
        }

        public async Task LoginAsync()
        {
            //Llamar Web Service
            var client = new HttpClient();

            string url = $"{_apiBase}Login?company={_company}&appid={_appid}";

            client.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Basic", GetAuthHeader(_email, _password));

            var jsonString = Newtonsoft.Json.JsonConvert.SerializeObject("");
            var content = new StringContent(jsonString, Encoding.UTF8, "application/json");

            var response = await client.PostAsync(url, content);

            if (response.StatusCode == System.Net.HttpStatusCode.Unauthorized)
            {
                var errorObject = new ApiResponse();
                errorObject.success = false;
                errorObject.message = "Usuario o clave inválidos";
                throw new ApplicationException(errorObject.message);
            }

            if (response.StatusCode != System.Net.HttpStatusCode.OK)
            {
                var apiBadResponseString = await response.Content.ReadAsStringAsync();
                var errorObject = new ApiResponse();
                errorObject.message = "Ha ocurrido un error " + response.StatusCode;
                throw new ApplicationException(errorObject.message);
            }

            var apiResponseString = await response.Content.ReadAsStringAsync();

            var returnedMsg = Newtonsoft.Json.JsonConvert.DeserializeObject<ApiResponse>(apiResponseString);

            if (!returnedMsg.success)
                throw new Exception(returnedMsg.message);
        }
    }
}
